#!/bin/sh



# ------------------------------------------
# Gets a registry setting

getsetting() 
{
  VALUE=`echo 'get conf!web!servers!'$SERVERNAME'!'$1 | regcmd -`
}


# ------------------------------------------
# checks that a module is enabled

moduleenabled()
{
    getsetting 'modules!'$1'!enabled' 2>/dev/null
    if [ $? -eq 0 ]
    then
	if [ "$VALUE" = "yes" ] 
	then
	    return 0
        fi
    fi
    return 1
}


# ------------------------------------------
# errors if a module isn't enabled

assert_moduleenabled()
{
    moduleenabled $1
    if [ $? -ne 0 ]
    then
       echo "FATAL: $1 module is not enabled for this server"
       exit;
    fi
    return 0
}


# ------------------------------------------
# check a UNIX user exits

checkuser()
{
  if [ "$1" = "" ] 
  then
	return 1
  fi
  TMPFILE=/tmp/zeusfptmp.$$
  touch $TMPFILE
  chown $1 $TMPFILE
  RET=$?  
  rm -f $TMPFILE
  return $RET
}


# ------------------------------------------
# Check a group exists

checkgroup()
{
  if [ "$1" = "" ] 
  then
	return 1
  fi
  TMPFILE=/tmp/zeusfptmp.$$
  touch $TMPFILE
  chgrp $1 $TMPFILE
  RET=$?  
  rm -f $TMPFILE
  return $RET
}


# ------------------------------------------
# gets and confirms a password from the user

getpassword()
{
 prompt=$1
 VALUE=""
 until [ "$VALUE" != "" ]
 do
    printf "$prompt"
    stty -echo
    read password1
    stty echo
    echo
    printf "Re-enter $prompt"
    stty -echo
    read password2
    stty echo
    echo 
    if [ "$password1" = "$password2" ]
    then
        VALUE=$password1
    fi
 done

 return 1
}


# ------------------------------------------
# checks a server exists

checkserver()
{
  a=`echo 'get conf!web!servers!'$1'!port' | regcmd - 2> /dev/null`
  if [ -z "$a" ]
  then
     return 1
  fi
  return 0
}



# ------------------------------------------
# when given a subserver name, will return the extra bit added the web
# server docroot

# sets $SUBROOT on return. Won't include leading slash
    
get_subroot() 
{
    SUBSERVERNAME=$1

    # work out the layout
    getsetting 'modules!subserver!layout'
    case "$VALUE" in
    '1')
        SNAME=$SUBSERVERNAME
	# strip off the leading www if its there
	echo $SNAME | grep "^www\." > /dev/null
	if [ $? -eq 0 ]
	then
	    SNAME=`echo $SNAME| cut -d'.' -f2`
	fi
	FIRSTLETTER=`echo $SNAME | cut -c1`
	SECONDLETTER=`echo $SNAME | cut -c2`
	if [ -z "$SECONDLETTER" ]
	then 
	    FIRSTLETTER="a"; SECONDLETTER="a"
	fi
	SUBROOT="$FIRSTLETTER/$SECONDLETTER/$SUBSERVERNAME"
	;;
    '2')
	SUBROOT="`subserver_hash.pl $SUBSERVERNAME`"
	;;
    *) 
	SUBROOT="$SUBSERVERNAME"
	;;
    esac
}

# ------------------------------------------

have_local_config()
{
  # get the virtual server name
  while [ 1 ]
  do
	printf "Enter virtual server name: "
	read SERVERNAME
	checkserver $SERVERNAME
	if [ $? -eq 0 ] 
	then
	  break
	fi
	printf "Invalid server name\n\n"
  done


  # get registry settings
  getsetting port; PORT=$VALUE
  getsetting ip_name; HOSTNAME=$VALUE
  getsetting docroot; DOCROOT=$VALUE

  # check the necessary modules are enabled
  assert_moduleenabled frontpage
  assert_moduleenabled htaccess 

  # if the subserver module is enabled, then we'll ask them which softvserver
  # they want to add frontpage to and alter the docroot to match

  moduleenabled subserver

  if [ $? -eq 0 ]
  then
    echo "This server has subserver support enabled."
	   
    while [ 1 ]
    do
	printf "Which subserver do you wish to enable FrontPage for? : "
	read SUBSERVERNAME
	get_subroot $SUBSERVERNAME

	if [ -d ${DOCROOT}/${SUBROOT} ] 
	then
	    break
	fi

	printf "Subserver does not exist (no directory ${DOCROOT}/${SUBROOT})\n\n"
    done
    FPSERVERNAME=${SERVERNAME}_${SUBSERVERNAME}
    HOSTNAME=$SUBSERVERNAME
    DOCROOT=${DOCROOT}/${SUBROOT}
  else
    # Not using subserver, FPSERVERNAME is the same as SERVERNAME
    FPSERVERNAME=$SERVERNAME
  fi
}


# ------------------------------------------
# The main code

cat << EOM

*****************************************
Zeus Technology - Express FrontPage Setup
Copyright (C) - Zeus Technology, 1999
Updated to work with FrontPage 2000.
*****************************************

EOM



# set a PATH that will contain all our tools
PATH=$ZEUSHOME/webadmin/bin:$ZEUSHOME/web/bin:/usr/local/frontpage/currentversion/bin:$PATH
export PATH
 
# check frontpage directory exists
if [ ! -d /usr/local/frontpage ]
then
  echo "FATAL: Microsoft FrontPage Extensions are not installed under "
  echo "/usr/local/frontpage"
  exit
fi

# check we have permissions to write to that dir
touch /usr/local/frontpage/z_touch_check 2> /dev/null
if [ $? -ne 0 ]
then
  echo "FATAL:  this script needs to be run as root"
  exit
fi
rm -f /usr/local/frontpage/z_touch_check

# check ZEUSHOME is set
if [ -z "$ZEUSHOME" ]
then
  echo "Please set \$ZEUSHOME to point to the directory under which Zeus applications are installed"
  exit 1
fi

if [ ! -d "$ZEUSHOME" ]
then
  echo "ERROR: \$ZEUSHOME is set to $ZEUSHOME, which doesn't exist"
fi

if [ -d $ZEUSHOME/webadmin ]
then
    have_local_config
else

cat <<EOF
ERROR: this script should only be run on the admin server machine. In
a clustered environment, you should share /usr/local/frontpage across
a common filesystem.
EOF
    exit 1
fi



# This might seem a tad severe, but the number of times customers have had
# problems with permissions means we should really try and help them out!

# Ensure permissions in /usr/local/frontpage are reasonable
# Everything should be readable
find /usr/local/frontpage/ -print | xargs chmod +r 
# Anything that is executable, ensure it is also world executable
find /usr/local/frontpage/ -perm -100 -print | xargs chmod +x



while [ 1 ] 
do
	printf "\nEnter UNIX user that owns the web: "
	read XUSER	
	checkuser $XUSER
	if [ $? -eq 0 ] 
	then
	  break
	fi
	printf "Invalid user name\n\n"
done


while [ 1 ]
do
	printf "Enter UNIX group that owns the web: "
	read XGROUP
	checkgroup $XGROUP
	if [ $? -eq 0 ] 
	then
	  break
	fi
	printf "Invalid group name\n\n"
done

echo

ADMIN=""
while [ -z "$ADMIN" ]
do
	printf "Enter the frontpage admin user name for the root web: "
	read ADMIN
done


PASSWD=""
getpassword "Enter password for the administrator: "
PASSWD=$VALUE


# OK - finally we have all the information we require.  Lets do some work

echo "Creating the FrontPage configuration files"

echo
ZFP=/usr/local/frontpage/zeus

# write out srm.conf
mkdir -p $ZFP/$FPSERVERNAME
#chown $XUSER $ZFP/$FPSERVERNAME
#chgrp $XGROUP $ZFP/$FPSERVERNAME
chmod 755 $ZFP
chmod 755 $ZFP/$FPSREVERNAME
echo "  Created directory $ZFP/$FPSERVERNAME"


# check the docroot exists
if [ ! -d "$DOCROOT" ]
then
    echo "Document Root $DOCROOT does not exist"
    exit
fi

# if its a symlink we must follow that symlink so that FP doesn't fail
# its security tests.  It won't ever write a file that is outside this 
# directory
if [ -h "$DOCROOT" ]
then
    NEWROOT=`ls -l "$DOCROOT" | sed 's|\(.*\)-> \(.*\)|\2|'`
    echo "Followed symlink from $DOCROOT to $NEWROOT"
    DOCROOT=$NEWROOT
fi


# write out a dumy srm.conf file that gives enough information about
# us so that the FrontPage CGIs know where we are

SRMCONF=$ZFP/$FPSERVERNAME/srm.conf
echo "DocumentRoot $DOCROOT" > $SRMCONF
echo "Port $PORT" >> $SRMCONF
echo "ServerRoot $ZFP/$FPSERVERNAME" >> $SRMCONF
echo '<VirtualHost '$HOSTNAME'>' >>  $SRMCONF
echo '</VirtualHost>' >>  $SRMCONF
# I don't think we need this anymore
# chown $XUSER $SRMCONF
# chgrp $XGROUP $SRMCONF
chmod 644 $ZFP/$FPSERVERNAME/srm.conf
echo "  Created file $ZFP/$FPSERVERNAME/srm.conf"


mkdir -p $ZFP/$FPSERVERNAME/etc
chmod 755 $ZFP/$FPSERVERNAME/etc
touch $ZFP/$FPSERVERNAME/etc/access.conf
chmod 644 $ZFP/$FPSERVERNAME/etc/access.conf
echo "  Created file $ZFP/$FPSERVERNAME/etc/access.conf"


# create the symbolic link

cd $ZFP/$FPSERVERNAME
ln -sf . conf 2>/dev/null
echo "  Created symlink $ZFP/$FPSERVERNAME/conf -> $ZFP/$FPSERVERNAME"



# run the Microsoft program to actually do the work.  But avoid their 
# interface.
# This program puts all the _vti files into the docroot.

echo
echo "Running Microsoft fpsrvadm program:"
fpsrvadm.exe -o install -p $PORT -s $SRMCONF -u $ADMIN -pw $PASSWD -type apache-fp -xUser $XUSER -xGroup $XGROUP -multihost $HOSTNAME

if [ $? -eq 0 ]
then
  echo
  echo "Installation Complete"
  echo 
  exit 0
else
  exit 1
fi



