#!/bin/sh

# This script starts a named virtual server from the command line
# It's designed for ISPs who might need to create a lot of servers


usage()
{
  echo
  echo "Starts a virtual server"
  echo "Usage: $0 servername [passwd]"
  echo
  exit
}



getpassword()
{
 prompt=$1
 printf "$prompt"
 stty -echo
 read VALUE
 stty echo
 echo
}


if [ $# -ne 1  -a $# -ne 2  ]
then
  usage
fi

# Take the arguments
NAME=$1
PASSWD=$2


# check ZEUSHOME is set
if [ -z "$ZEUSHOME" ]
then
  echo "Please set \$ZEUSHOME to point to the directory under which Zeus applications are installed"
  exit 1
fi


if [ ! -d $ZEUSHOME/webadmin ]
then
    echo "This script should only be run on machines with the webadmin package on"
    exit 1
fi



PATH=$ZEUSHOME/admin/bin:$PATH
cd $ZEUSHOME

reset()
{
  stty echo
  exit
}

# if we don't have a password from the command line, then prompt for one
if [ -z "$PASSWD" ]
then
  trap reset INT
  getpassword "Enter admin passwd: "
  PASSWD=$VALUE
fi


HOST=`hostname`
echo $HOST | grep '\.' > /dev/null
if [ $? -ne 0 ]
then
  DOMAIN=`grep domain /etc/resolv.conf | cut -d' ' -f2`
  HOST=${HOST}.${DOMAIN}
  echo $HOST | grep '\.' > /dev/null
  if [ $? -ne 0 ]
  then
     echo FATAL: cannot work out full hostname
     exit 1
  fi
fi


PORT=`echo 'get conf!admin!port' | webadmin/bin/regcmd -`

res=`httpclient http://${HOST}:${PORT}/apps/web/website.zap?website=$NAME\&action=start -auth admin:$PASSWD`


echo $res | grep  '401 Authenticate' > /dev/null
if [ $? -eq 0 ]
then
  echo "Invalid password"
  exit 1 
fi

echo $res | grep "No active webserver process" > /dev/null
if [ $? -eq 0 ]
then
  echo "Web Server is not running"
  exit 1
fi

echo $res | grep "already running" > /dev/null
if [ $? -eq 0 ]
then
  echo "Server is already running"
  exit 1
fi

if [ -f $ZEUSHOME/webadmin/conf/runningsites/$NAME ]
then
  echo Startup succeeded
  exit 0
fi

echo "startup failed"
exit 1



