<html>
<head>
<title>Cluster status</title>
<% 
#
# Like status.zap, but doesn't know about Zeus cluster frontends etc.
# Just a simple load of backend machines.
#

#############################################################################
# Check for auto-refresh
set refresh 0
asp::decode_input
if {[info exists asp::input(refresh)] && $asp::input(refresh)=="yes" } {
    set refresh 1
    asp::write "<meta http-equiv=Refresh content=\"2; $request::env(SCRIPT_NAME)?refresh=yes\">"
}


#############################################################################
## Attempts to connect with a timeout, either errors or returns a 
## socket channel (blocking)
##
proc connect_timeout { host port timeout } {
    global connect_timeout ;# global table for our state info
    set connect_timeout(connected) ""
    set connect_timeout(socket) [socket -async $host $port]
    set t [expr $timeout*1000]
    set connect_timeout(timeoutid) [after $t { 
	close $connect_timeout(socket)
	set connect_timeout(connected) "no"
    }]
    fileevent $connect_timeout(socket) writable { 
	after cancel $connect_timeout(timeoutid)
	set connect_timeout(connected) "yes"
    }
    # Now wait if we didn't manage it straight away....
    if { $connect_timeout(connected) == "" } { 
	vwait connect_timeout(connected) 
    }
    if { $connect_timeout(connected) == "yes" } { 
	# Ensure we are back blocking again
	fconfigure $connect_timeout(socket) -blocking yes 
	return $connect_timeout(socket) 
    }
    error "Connect timeout"
}

#############################################################################
##
## Draws a pictorial representation of the current server status
##
## Requries
##    zap functionality
##    single-black.gif
##    server.gif
##    deadserver.gif

proc draw_status {} {

    #
    ## Registry settings
    #
    # List of backend machines (host:port)
    set hostsfile "$request::env(ZEUSHOME)/webadmin/conf/hosts"

    # ----------------------------------------------------------
    # Returns 1 if the backend machine is functional, else 0
    # A backend machine is considered functional if its response time < 10000
    
    proc isworking { backend } {
	# Just see if we can connect to it at all
	regexp (.*):(\[0-9]+) $backend tmp host port
	# Try to connect to host:port
	set ok 0
	catch {
	    set s [ connect_timeout $host $port 2 ]
	    puts $s "info\n"; flush $s
	    set tmp [ read $s ]
	    close $s
	    set ok 1
	}
	return $ok
    }

    # ----------------------------------------------------------
    # Returns the response time in ms
    proc getstats { backend } {
	# TODO
	return "";
    }
    
    # ----------------------------------------------------------
    # Returns the hostname of a machine given its full name
    
    proc hostname { fullname } {
	return [lindex [split $fullname .] 0]
    }
    
    # ----------------------------------------------------------
    
    ## Get the settings
    set f [open $hostsfile]
    set backends [lsort -dictionary [read $f]]
    close $f
    

    ## Now lets think about our table
    #
    # |-------------------------------------------------|
    # |backend1|...|backend2|....|backend3|....|backend4|
    # ---------------------------------------------------
    
    # Work out how many columns we need
    set columns [llength $backends]

    # Start our table
    asp::write "<table border=0 cellspacing=0 cellpadding=0>\n"
    
    # Write out the columns
    asp::write "<tr>"
    for {set i 0} {$i < $columns} {incr i} {asp::write "<td>&nbsp;</td>" }
    asp::write "</tr>\n"

    # We need a router!
    asp::write "<tr><td colspan=$columns align=middle>
    <img src=router.gif><br>
    <img src=single-black.gif height=20 width=1></td></tr>\n"

    # Now our bus
    asp::write "<tr bgcolor=#000000><td height=1 colspan=$columns>
    <img src=single-black.gif width=1 height=1></td></tr>\n"

    # Now the backend onto bus row
    asp::write "<tr>"
    foreach i $backends {
	# Check if this backend is working
	set msg "&nbsp;"
	if { [isworking "$i"]  } { 
	    set msg "<img src=single-black.gif height=20 width=1>" 
	}
	asp::write "<td align=middle>$msg</td>"
    }
    
    asp::write "</tr>\n"
    
    # Now the backend row
    asp::write "<tr>"
    foreach i $backends {
	set img "server.gif"
	if { ![isworking "$i"] } { set img "deadserver.gif" }
	asp::write "<td align=middle valign=top>
	<img src=$img hspace=30><br>
	<font size=-1>[hostname $i]<br>
	[getstats \"$i\"]</font></td>"
    }
    asp::write "</tr>\n"
    
    # All done
    asp::write "</table>"
}
##############################################################################

%>
</head>

<body bgcolor="#ffffff" link=#990066 vlink=#990066 alink=#990066>

<table>
<tr>
 <td valign=top>
  <img src="/apps/web/title.gif" hspace=10 alt="">
  <p align=right>

  <a href="/apps/admin/docsystem/context.asp?help=/apps/web/docs/controller/cluster.html" target="help" onClick=
     "window.open('','help','resizable=1,width=640,height=500'); return true;">
        <img src="/apps/web/docs.gif" 
        alt="[documentation]" align=bottom border=0 hspace=0></a>

  <a href="/apps/web/"><img src="/apps/web/back.gif" 
        alt="[back]" align=bottom border=0 hspace=0></a>
        <a href="/apps/web/"><img src="/apps/web/home.gif" alt="[Controller]" 
        align=bottom border=0></a>
  </td>
  <td valign=top>


<br>
<img src="title-clusterstatus.gif" alt="Cluster Status">

<p>

Zeus Web Server has been designed to easily manage a cluster of
machines with a common configuration.  This page lets you view and
configure your cluster setup.  Click on the book icon for more
information about setting up a cluster.

<p>

<ul>
<%
draw_status
asp::write "<p>"

if { $refresh } {
    asp::write "<a href=\"$request::env(SCRIPT_NAME)\">
    Disable auto-refresh</a>"
} else {
    asp::write "<a href=\"$request::env(SCRIPT_NAME)?refresh=yes\">
    Enable auto-refresh</a>"
}
%>
&nbsp;&nbsp;&nbsp;&nbsp;
<a href="settings.zap">Configuration</a>
</ul>

</td>
</tr>
</table>

</body>
</html>
