<%
## Adds a new website to the webserver
## If successful, redirects to controller, else displays error
##
## Arguments
##    name 
##    ip_name  (server:port)
##    docroot
##    clone  (!default means skel)
##

# Returns non-zero if the webserver is local
proc localwebserver {} {
    set hostsfile "$request::env(ZEUSHOME)/webadmin/conf/hosts"
    set f [open $hostsfile]
    set data [read $f]
    close $f

    if { [llength $data] != 1 } { return 0 }
    # split server into host:port
    regexp (.*):(\[0-9]+) $data tmp host port
    if { $host != "localhost" } { return 0 }
    return 1
}

# Decode and check the input before calling our add routine
zap::getrequest { name ip_name docroot }
set name    $asp::input(name)
set ip_name $asp::input(ip_name)
set docroot $asp::input(docroot)
set clone   $asp::input(clone)

if { [ string match "* *" $name ] } {
    zap::error "Server name may not contain spaces"
}

if { [ string match "* *" $ip_name ] } {
    zap::error "Website URL may not contain spaces"
}

# Process ip_name field, it may have a port number etc.
set port 80
regexp (.*):(\[0-9]+) $ip_name tmp ip_name port

# Check our setting are reasonable
if { $ip_name == "" }          { zap::error "Blank website URL" }
if { $port    == "" }          { zap::error "Blank port" }
if { $port    == "0" }         { zap::error "Invalid port" }
if { $name    == "" }          { zap::error "Blank name" }

# If we are clustering, we don't check if the docroot exists, as it
# may only exist on the back-end machines...
if { [ localwebserver ] } {
    if { [file exists $docroot] == 0 } {
	zap::error "Document root directory <b>$docroot</b> does not exist" 
    }
    if { [file isdirectory $docroot ] == 0 } {
	zap::error "Document root <b>$docroot</b> is not a directory!" 
    }
}

# Check if we already have one 
set filename "$request::env(ZEUSHOME)/webadmin/conf/sites/$name"
if { [ file exists $filename ] } {
    zap::error "Server <b>$name</b> already exists"
}

# Work out skeleton file
if { $clone != "!default" } {
    set skelfile "$request::env(ZEUSHOME)/webadmin/conf/sites/$clone"
} else {
    set skelfile "$request::env(ZEUSHOME)/webadmin/conf/skel"
}

# Copy skel file to new filename (makes it part of the configuration namespace)
file copy $skelfile $filename

# Can now update with registry commands...
reg::add "conf!web!servers!$name!ip_name" $ip_name text
reg::add "conf!web!servers!$name!port"    $port    integer
reg::add "conf!web!servers!$name!docroot" $docroot text

# It succceded
asp::redirect "/apps/web/"
%>