<%
# Starts/stops a website on all running webservers
#
# Fields required
#   website
#   action  (start|stop|restart)

# TODO: do all for clustering case, just does on one at the moment

zap::getrequest { website action }

set website $asp::input(website)
set action  $asp::input(action)
set Action  "Start"
if { $action == "stop" }    { set Action "Stop" }
if { $action == "restart" } { set Action "Restart" }

global secret
set secret ""

## Debug line
proc debug { msg } {
#    asp::write "<pre>$msg</pre>"
}

## Sends a command to the given host:port
## On success, returns a string containing any info generated by this
## command.  Else throws an error with error info

proc command { host port cmd } {
    set data ""
    global secret

    if { [ catch { set s [ socket $host $port ] } ] } { 
	error "Cannot connect to webserver" };
    if { [ catch {
	set s [ socket $host $port ]
	puts -nonewline $s "${secret}$cmd"; flush $s
	set data [read $s]
    } ] } { error "Communication failure with webserver" }

    # Now process response from the webserver
    if { [llength $data] == 0 } { error "No response from webserver" }

    # We managed to get a result, now see what it is
    # Format is
    #  (Success|Failure)\n
    #  (body)\n*
    set lines     [split $data \n]
    set firstline [lindex $lines 0]
    set body      [join [lrange $lines 1 end] \n]

    if { $firstline == "Success" } {
	# It worked, return body if any
	debug "command worked returning ($body)"
	return $body
    } else {
	debug "command failed, returned ($body)"
	error "$body"
    }
}


## Starts a website, given a list of backend & frontend machines
## The machines lists are lists of tuples of (host:port)
## On success, returns a string containing any info generated by this
## command.  Else throws an error with error info

proc start { frontend backend website } {
    # We start on all the backends first.  If that works we start the
    # frontends

    set response ""
    set usefulresponse 0

    if { [llength $frontend] == 0 && [llength $backend] == 0 } {
	error "Webserver is not running, please start the webserver first"
    }

    set runningfilename "$request::env(ZEUSHOME)/webadmin/conf/runningsites/$website"

    # Copy the file into the running directory
    if { [ catch { file copy -force $request::env(ZEUSHOME)/webadmin/conf/sites/$website $runningfilename } msg ] } {
	error "Unable to copy file to running directory: $msg"
    }

    # Set the file to readonly
    file attributes $runningfilename -permissions 0400

    # Work out the data we want to send
    set sitedata [read [open $runningfilename ]]
    set startcmd "start $website/[string length $sitedata]\n$sitedata"

    foreach site $backend {
	set siteresponse ""
	set siteerror ""
	set host [lindex $site 0]
	set port [lindex $site 1]

	if { [ catch { set siteresponse \
	[command $host $port $startcmd] } siteerror ] } {
	    append response "<b>Errors on backend machine $host</b>\n"
	    append response "<ul><pre>$siteerror</pre></ul>\n"
	    error $response
	}
	append response "<b>Successful startup on backend machine $host</b>\n"
	append response "<ul><pre>$siteresponse</pre></ul>\n"
	if { $siteresponse != "" } { incr usefulresponse }
    }

    # All backend machines done successfully, now start frontend machines
    foreach site $frontend {
	set siteresponse ""
	set siteerror ""
	set host [lindex $site 0]
	set port [lindex $site 1]

	if { [ catch { set siteresponse \
	[command $host $port $startcmd] } siteerror ] } {
	    append response "<b>Errors on frontend machine $host</b>\n"
	    append response "<ul><pre>$siteerror</pre></ul>\n"
	    error $response
	}
	append response "<b>Successful startup on frontend machine $host</b>\n"
	append response "<ul><pre>$siteresponse</pre></ul>\n"
	if { $siteresponse != "" } { incr usefulresponse }
    }

    # Everything worked
    if { $usefulresponse == 0 } { set response "" }
    return $response
}


## Stops a website, given a list of backend & frontend machines
## The machines lists are lists of tuples of (host:port)
## On success, returns a string containing any info generated by this
## command.  Else throws an error with error info

proc stop { frontend backend website } {
    # We stop all the frontends first. If that works we stop the backends

    set response ""
    set usefulresponse 0

    if { [llength $frontend] == 0 && [llength $backend] == 0 } {
	error "Webserver is not running"
    }

    set stopcommand "stop $website\n"


    foreach site $frontend {
	debug "frontend stop $site"
	set siteresponse ""
	set siteerror ""
	set host [lindex $site 0]
	set port [lindex $site 1]

	if { [ catch { set siteresponse \
	[command $host $port $stopcommand] } siteerror ] } {
            debug "error response from server ($siteerror)"
	    append response "<b>Errors on frontend machine $host</b>\n"
	    append response "<ul><pre>$siteerror</pre></ul>\n"
	    error $response
	}
	debug "success response from server ($siteresponse)"
	append response "<b>Successful stop on frontend machine $host</b>\n"
	append response "<ul><pre>$siteresponse</pre></ul>\n"
	if { $siteresponse != "" } { incr usefulresponse }
    }

    # All frontend machines done successfully, now start backend machines
    foreach site $backend {
	debug "backend stop $site"
	set siteresponse ""
	set siteerror ""
	set host [lindex $site 0]
	set port [lindex $site 1]

	if { [ catch { set siteresponse \
	[command $host $port $stopcommand] } siteerror ] } {
	    append response "<b>Errors on backend machine $host</b>\n"
	    append response "<ul><pre>$siteerror</pre></ul>\n"
	    error $response
	}
	append response "<b>Successful stop on backend machine $host</b>\n"
	append response "<ul><pre>$siteresponse</pre></ul>\n"
	if { $siteresponse != "" } { incr usefulresponse }
    }

    # Everything worked
    if { $usefulresponse == 0 } { set response "" }
    return $response
}


## Returns a list of tuples of host,port of the frontend machines
proc frontends {} {
    # TODO: currently no frontends, that's part of the seperate clustering
    # software
    set result {}
    return $result
}

## Returns a list of tuples of host,port of the the backend machines
proc backends {} {
    set hostsfile "$request::env(ZEUSHOME)/webadmin/conf/hosts"
    set f [open $hostsfile]
    set data [read $f]
    close $f

    set result {}
    
    foreach server $data {
	regexp (.*):(\[0-9]+) $server tmp host port
	lappend result [ list $host $port ]
    }
    return $result
}

#############################################################################
# Attempts to connect with a timeout, either errors or returns a 
# socket channel (blocking)
#
proc connect_timeout { host port timeout } {
    global connect_timeout ;# global table for our state info
    set connect_timeout(connected) ""
    set connect_timeout(socket) [socket -async $host $port]
    set t [expr $timeout*1000]
    set connect_timeout(timeoutid) [after $t { 
	close $connect_timeout(socket)
	set connect_timeout(connected) "no"
    }]
    fileevent $connect_timeout(socket) writable { 
	after cancel $connect_timeout(timeoutid)
	set connect_timeout(connected) "yes"
    }
    # Now wait if we didn't manage it straight away....
    if { $connect_timeout(connected) == "" } { 
	vwait connect_timeout(connected) 
    }
    if { $connect_timeout(connected) == "yes" } { 
	# Ensure we are back blocking again
	fconfigure $connect_timeout(socket) -blocking yes 
	return $connect_timeout(socket) 
    }
    error "Connect timeout"
}

# Go do it
set response ""
set responseerr ""

if { $action == "start" } {
    if { [ catch { 
	set response [start [frontends] [backends] $website ] 
    } responseerr ] } {
	# Startup failed, shut it all done
	catch { stop [frontends] [backends] $website }
	set response "<h3>Startup failure for $website</h3>\n$responseerr"
	# Remove site from running directory
	file delete "$request::env(ZEUSHOME)/webadmin/conf/runningsites/$website"
    } else {
	# Startup succeeded
	if { $response != "" } { 
	    set response "<h3>Startup success for $website</h3>\n$response"
	}
    }
} elseif { $action == "stop" } {
    if { [ catch { 
	set response [stop [frontends] [backends] $website ] 
    } responseerr ] } {
	# Stop failed
	set response "<h3>Stop failure for $website</h3>\n$responseerr"
    } else {
	# Stop succeeded
	if { $response != "" } {
	    set response "<h3>Stop success for $website</h3>\n$response"
	}
	# Remove site from running directory
	file delete "$request::env(ZEUSHOME)/webadmin/conf/runningsites/$website"
    }
} elseif { $action == "restart" } {
    # Do the stop code first, if the stop fails because we're not 
    # currently running, then ignore the stop failure and carry on
    if { [ catch { 
	set response [stop [frontends] [backends] $website ] 
    } responseerr ] } {
	# Stop failed
	set response "<h3>Stop failure for $website</h3>\n$responseerr"
    } else {
	# Stop succeeded
	# Remove site from running directory
	file delete "$request::env(ZEUSHOME)/webadmin/conf/runningsites/$website"
    }

    # Stop succeeded, now do start code
    if { [ catch { 
	set response [start [frontends] [backends] $website ] 
    } responseerr ] } {
	# Startup failed, shut it all done
	catch { stop [frontends] [backends] $website }
	set response "<h3>Startup failure for $website</h3>\n$responseerr"
	# Remove site from running directory
	file delete "$request::env(ZEUSHOME)/webadmin/conf/runningsites/$website"
    } else {
	# Startup succeeded
	if { $response != "" } {
	    set response "<h3>Startup success for $website</h3>\n$response"
	}
    }
}

if { $response == "" } {
    # Nothing useful to say, lets redirect straight away
    # If there is a referer send them back to that, otherwise send them
    # to the first page for the traffic light page.
    set back "/apps/web"
    catch { set back $request::env(HTTP_REFERER) }
    asp::redirect $back
}

%>

<html>
<head><title><% asp::write "$Action website $website" %></title></head>
<body bgcolor=#ffffff>
   <h1><% 
        if { $Action == "Stop" } { asp::write "Stopp" 
        } else { asp::write "$Action" }
       %>ing website <%= $website %></h1>
   <%= $response %>
<p>
<a href="/apps/web">Back to controller</a>
</body>
</html>
